module JavaParser where

import Data.Maybe
import JavaLexer
import Kombinatoren

data Stmt = If(BExpr, Stmt , Maybe Stmt)
          | While( BExpr , Stmt )
          | Block([Stmt])
          | Empty
          | Return( Maybe Expr )
          deriving (Show)

data BExpr = T
     deriving (Show)

data Expr = One
     deriving (Show)

isInt m (INTLITERAL n) = n == m
isInt _ _ = False

isBool v (BOOLLITERAL w) = v == w
isBool _ _ = False



stmt :: Parser Token Stmt
stmt = ifstmt 
     ||| ifelsestmt
     ||| whilestmt
     ||| block
     ||| emptystmt
     ||| returnstmt

ifstmttransform (_, (_, (bexpr, (_, s)))) = If(bexpr, s, Nothing)
ifstmt :: Parser Token Stmt
ifstmt = ((lexem IF) +.+ (lexem LBRACE) +.+ bexpression 
       +.+ (lexem RBRACE) +.+ stmt) 
       <<< ifstmttransform -- (\(_, (_, (bexpr, (_, s)))) -> If(bexpr, s, Nothing))

ifelsestmt :: Parser Token Stmt
ifelsestmt = ((lexem IF) +.+ (lexem LBRACE) +.+ bexpression 
       +.+ (lexem RBRACE) +.+ stmt +.+ (lexem ELSE) +.+ stmt)
       <<< (\(_, (_, (bexpr, (_, (s1, (_, s2)))))) 
           -> If(bexpr, s1, Just s2))

whilestmt :: Parser Token Stmt
whilestmt = ((lexem WHILE) +.+ (lexem LBRACE) +.+ bexpression 
       +.+ (lexem RBRACE) +.+ stmt) 
       <<< (\(_, (_, (bexpr, (_, s)))) -> While(bexpr, s))

block :: Parser Token Stmt
block = (((lexem LBRACKET) +.+ (lexem RBRACKET)) 
      <<< (\(_, _) -> Block([])))
      ||| (((lexem LBRACKET) +.+ stmts +.+ (lexem RBRACKET))
          <<< (\(_, (s, _)) -> Block(s)))

stmts :: Parser Token [Stmt]            --Linksrekursion aufgeloest
stmts = ((stmt +.+ stmts) <<< (\(s, sts) -> (s : sts)))
      ||| (stmt <<< \s -> [s])

emptystmt :: Parser Token Stmt
emptystmt = (lexem SEMICOLON) <<< (\_ -> Empty)

returnstmt :: Parser Token Stmt
returnstmt = (((lexem RETURN) +.+ (lexem SEMICOLON)) 
     <<< (\(_, _) -> Return Nothing))
     ||| (((lexem RETURN) +.+ expression +.+ (lexem SEMICOLON)) 
     <<< (\(_, (e, _)) -> Return (Just e)))
     


bexpression = (satisfy (isBool True)) <<< (\_ -> T)
expression = (satisfy (isInt 1)) <<< (\_ -> One)

-- nur wenn keine Tokens brig sind ist die Loesung korrekt
correctsols :: [(t, [a])] -> [(t, [a])]
correctsols sols = (filter (\(_, resttokens) -> null resttokens)) sols

parser :: String -> Stmt
parser eingabe = fst(head(correctsols(stmt(alexScanTokens(eingabe)))))

main = do
  s <- readFile "fst.stmt"
  --s <- getContents
  print (parser s)
